/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.dao;

import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;

import org.apache.log4j.Logger;

public final class DAOFactory {

	private DAOFactory() {
	}

	private static transient Logger logger = Logger.getLogger(DAOFactory.class);

	public static Object getDAO(final String daoEnvEntry) throws EMFUserError {
		logger.debug("Begin Istantiation of DAO [" + daoEnvEntry + "]");
		Object daoObject = null;
		try {
			ConfigSingleton configSingleton = ConfigSingleton.getInstance();
			SourceBean daoConfigSourceBean = (SourceBean) configSingleton
					.getFilteredSourceBeanAttribute("SPAGO4Q.DAO-CONF.DAO",
							"name", daoEnvEntry);
			String daoClassName = (String) daoConfigSourceBean
					.getAttribute("implementation");
			logger.debug("DAO [" + daoEnvEntry + "] Implementation class ["
					+ daoClassName + "]");

			daoObject = Class.forName(daoClassName).newInstance();
			logger.debug("createDAOInstance DAO [" + daoEnvEntry
					+ "] Instatiate successfully");
		} catch (InstantiationException e) {
			throw new EMFUserError(EMFErrorSeverity.ERROR, 1000);
		} catch (IllegalAccessException e) {
			throw new EMFUserError(EMFErrorSeverity.ERROR, 1000);
		} catch (ClassNotFoundException e) {
			throw new EMFUserError(EMFErrorSeverity.ERROR, 1000);
		}
		return daoObject;
	}

}
