/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors.scheduler.job;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.OperationCoordinatorFactory;
import it.eng.spago4q.extractors.OperationCoordinatorInterface;

import org.apache.log4j.Logger;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

/**
Non-stateful jobs only have their JobDataMap stored at the time they are added to the scheduler.
This means that any changes made to the contents of the job data map during execution of the job will be lost,
and will not seen by the job the next time it executes
*/

public class ProcessExtractionJob implements Job {

	private static transient Logger logger = Logger
			.getLogger(ProcessExtractionJob.class);

	public void execute(final JobExecutionContext context)
			throws JobExecutionException {
		logger.debug(ProcessExtractionJob.class.getName() + "::execute::"
				+ "Begin");
		System.out.println("Begin execution of ProcessExtractionJob");
		JobDataMap dataMap = context.getJobDetail().getJobDataMap();
		int eProcessId = dataMap.getInt("E_PROCESS_ID");

		logger.debug(ProcessExtractionJob.class.getName() + "::execute::"
				+ "Begin Istantiation of Operation Coordinetor"
				+ "of extraction process [" + eProcessId + "]");
		try {
			OperationCoordinatorInterface operationCoordinator =
				OperationCoordinatorFactory.create(eProcessId);

			operationCoordinator.run();

			logger.debug(ProcessExtractionJob.class.getName() + "::execute::"
					+ "End execution of extraction process [" + eProcessId
					+ "]");
			System.out.println("End execution of ProcessExtractionJob");

		} catch (EMFUserError e) {
			logger.warn(ProcessExtractionJob.class.getName()
					+ "::execute::An error Occurs durring the execution"
					+ "of extraction process [" + eProcessId + "]", e);
		}

	}

}
